package java.util.stream;

import java.util.Iterator;
import java.util.function.IntSupplier;
import java.util.Objects;
import java.util.Arrays;
import java.util.Spliterators;
import java.util.Spliterator;
import java.util.PrimitiveIterator;
import java.util.IntSummaryStatistics;
import java.util.OptionalDouble;
import java.util.function.BiConsumer;
import java.util.function.ObjIntConsumer;
import java.util.function.Supplier;
import java.util.OptionalInt;
import java.util.function.IntBinaryOperator;
import java.util.function.IntConsumer;
import java.util.function.IntToDoubleFunction;
import java.util.function.IntToLongFunction;
import java.util.function.IntFunction;
import java.util.function.IntUnaryOperator;
import java.util.function.IntPredicate;

public interface IntStream extends BaseStream<Integer, IntStream>
{
    public abstract IntStream filter(IntPredicate p0);
    
    public abstract IntStream map(IntUnaryOperator p0);
    
    public abstract <U> Stream<U> mapToObj(IntFunction<? extends U> p0);
    
    public abstract LongStream mapToLong(IntToLongFunction p0);
    
    public abstract DoubleStream mapToDouble(IntToDoubleFunction p0);
    
    public abstract IntStream flatMap(IntFunction<? extends IntStream> p0);
    
    public abstract IntStream distinct();
    
    public abstract IntStream sorted();
    
    public abstract IntStream peek(IntConsumer p0);
    
    public abstract IntStream limit(long p0);
    
    public abstract IntStream skip(long p0);
    
    public abstract void forEach(IntConsumer p0);
    
    public abstract void forEachOrdered(IntConsumer p0);
    
    public abstract int[] toArray();
    
    public abstract int reduce(int p0, IntBinaryOperator p1);
    
    public abstract OptionalInt reduce(IntBinaryOperator p0);
    
    public abstract <R> R collect(Supplier<R> p0, ObjIntConsumer<R> p1, BiConsumer<R, R> p2);
    
    public abstract int sum();
    
    public abstract OptionalInt min();
    
    public abstract OptionalInt max();
    
    public abstract long count();
    
    public abstract OptionalDouble average();
    
    public abstract IntSummaryStatistics summaryStatistics();
    
    public abstract boolean anyMatch(IntPredicate p0);
    
    public abstract boolean allMatch(IntPredicate p0);
    
    public abstract boolean noneMatch(IntPredicate p0);
    
    public abstract OptionalInt findFirst();
    
    public abstract OptionalInt findAny();
    
    public abstract LongStream asLongStream();
    
    public abstract DoubleStream asDoubleStream();
    
    public abstract Stream<Integer> boxed();
    
    public abstract IntStream sequential();
    
    public abstract IntStream parallel();
    
    public abstract PrimitiveIterator.OfInt iterator();
    
    public abstract Spliterator.OfInt spliterator();
    
    public static IntStream.Builder builder() {
        return new Streams.IntStreamBuilderImpl();
    }
    
    public static IntStream empty() {
        return StreamSupport.intStream(Spliterators.emptyIntSpliterator(), false);
    }
    
    public static IntStream of(int n) {
        return StreamSupport.intStream(new Streams.IntStreamBuilderImpl(n), false);
    }
    
    public static IntStream of(int... array) {
        return Arrays.stream(array);
    }
    
    public static IntStream iterate(final int n, final IntUnaryOperator obj) {
        Objects.requireNonNull(obj);
        return StreamSupport.intStream(Spliterators.spliteratorUnknownSize(new PrimitiveIterator.OfInt() {
            int t = n;
            
            @Override
            public boolean hasNext() {
                return true;
            }
            
            @Override
            public int nextInt() {
                int t = this.t;
                this.t = obj.applyAsInt(this.t);
                return t;
            }
        }, 1296), false);
    }
    
    public static IntStream generate(IntSupplier obj) {
        Objects.requireNonNull(obj);
        return StreamSupport.intStream(new StreamSpliterators.InfiniteSupplyingSpliterator.OfInt(Long.MAX_VALUE, obj), false);
    }
    
    public static IntStream range(int n, int n2) {
        if (n >= n2) {
            return empty();
        }
        return StreamSupport.intStream(new Streams.RangeIntSpliterator(n, n2, false), false);
    }
    
    public static IntStream rangeClosed(int n, int n2) {
        if (n > n2) {
            return empty();
        }
        return StreamSupport.intStream(new Streams.RangeIntSpliterator(n, n2, true), false);
    }
    
    public static IntStream concat(IntStream obj, IntStream obj2) {
        Objects.requireNonNull(obj);
        Objects.requireNonNull(obj2);
        return (IntStream)StreamSupport.intStream(new Streams.ConcatSpliterator.OfInt(obj.spliterator(), obj2.spliterator()), obj.isParallel() || obj2.isParallel()).onClose(Streams.composedClose(obj, obj2));
    }
    
    public interface Builder extends IntConsumer
    {
        public abstract void accept(int p0);
        
        default public IntStream.Builder add(int n) {
            this.accept(n);
            return this;
        }
        
        public abstract IntStream build();
    }
}
